#| theme.jl for t:o)m's new theme design

   Copyright (C) 2001 Thomas "t:o)m" Frank

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   $Id: theme.jl,v 0.64 2001/12/24 13:11:24 jsh Exp $

   Author: Thomas "t:o)m" Frank <tom@madguru.de>

|#

(require 'rep.data.tables)		;need hash tables for icon cache
(require 'sawfish.wm.util.recolor-image)
(require 'sawfish.wm.util.gtkrc)

;; Customization

(defgroup aCOW "aCOW Theme"
  :group appearance)

(defcustom aCOW:text-justify 'left
  "Text is \\w justified in window titles."
  :type (choice left right center)
  :group (appearance aCOW)
  :after-set after-setting-frame-option)

(define (text-justifier w)
  (case aCOW:text-justify
    ((left) 6)
    ((right) -8)
    ((center) 'center)))


(let*
;; Update window title pixel length
    ((title-width
      (lambda (w)
      (let ((w-width (car (window-dimensions w))))
	  (max 0 (min (- w-width 100) (text-width (window-name w))))))))

;; 21x24
(define top-left
  (list (make-image "i_top_left.png")
	(make-image "a_top_left.png")))

;; 21x18
(define top-title-left
  (list (make-image "i_top_title_left.png")
	(make-image "a_top_title_left.png")))

;; 21x18
(define top-title
  (list (make-image "i_top_title.png" )
	(make-image "a_top_title.png" )))

;; 21x18
(define top-title-right
  (list (make-image "i_top_title_right.png")
	(make-image "a_top_title_right.png")))

;; 21x18
(define top-right
  (list (make-image "i_top_right.png")
	(make-image "a_top_right.png")))

;; 21x18
(define top-over
  (list (make-image "a_over.png")
	(make-image "a_over.png")))

;; 5x16*
(define left-top
  (list (make-image "i_left_top.png")
	(make-image "a_left_top.png")))

;; 21x63
(define left-middle
  (list (make-image "i_left_middle.png")
	(make-image "a_left_middle.png")))

;; 5x16*
(define left-grow-top
  (list (make-image "i_left_grow_top.png")
	(make-image "a_left_grow_top.png")))

;; 11x9
(define left-grow
  (list (make-image "i_left_grow.png")
	(make-image "a_left_grow.png")))

;; 133x26
(define right-tail
  (list (make-image "i_right_tail.png")
	(make-image "a_right_tail.png")))

;; 6x16*
(define right-top
  (list (make-image "i_right_top.png")
	(make-image "a_right_top.png")))

;; 6x33
(define right-middle
  (list (make-image "i_right_middle.png")
	(make-image "a_right_middle.png")))

;; 6x33
(define right
  (list (make-image "i_right.png")
	(make-image "a_right.png")))

;; 6x29
(define right-grow-top
  (list (make-image "i_right_grow_top.png")
	(make-image "a_right_grow_top.png")))

;; 4x4
(define right-grow
  (list (make-image "i_right_grow.png")
	(make-image "a_right_grow.png")))

;; 1x1
(define bottom-line
  (list (make-image "dot.png")
	(make-image "dot.png")))

;; BUTTONs
(define button-exit-images (list 
    (make-image "i_button_exit.png")
    (make-image "a_button_exit.png")
    (make-image "a_button_exit_over.png")
    (make-image "a_button_exit.png")
    ))

;; MAX-Button
(define button-max-images (list 
    (make-image "i_button_max.png")
    (make-image "a_button_max.png")
    (make-image "a_button_max_over.png")
    (make-image "a_button_max.png")
    ))

;; MENU-Button
(define button-menu-images (list
    (make-image "i_button_menu.png")
    (make-image "a_button_menu.png")
    (make-image "a_button_menu_over.png")
    (make-image "a_button_menu.png")
    ))

;; ICON-Button
(define button-icon-images (list 
    (make-image "i_button_icon.png")
    (make-image "a_button_icon.png")
    (make-image "a_button_icon_over.png")
    (make-image "a_button_icon.png")
    ))

;; SHADE-Button
(define button-shade-images (list 
    (make-image "i_button_shade.png")
    (make-image "a_button_shade.png")
    (make-image "a_button_shade_over.png")
    (make-image "a_button_shade.png")
    ))


;; geometry computations

(define (title-left-width w)
  (max 0 (min (- (car (window-dimensions w))
		(text-width (window-name w)) 16
		))))

(define (title-right-width w)
  (max 0 (min (- (car (window-dimensions w))
		(text-width (window-name w))
		))))
    
(define (vertical-justification)
  ;; `center' justification adjusted for the 3-pixel top-border
  (max 0 (- (/ (- 25 (font-height default-font)) 2) 2)))


;; frames
(define common-frame-parts
  `(
    ((background . ,top-left)
     (left-edge . -17)
     (top-edge . -30)
     (class . top-left-corner))

    ((background . ,top-title)
     (left-edge . 12)
     (text . ,window-name)
     (font . ("-adobe-helvetica-medium-r-normal-*-10-*-*-*-p-*-iso10646-1" "-adobe-helvetica-bold-r-normal-*-10-*-*-*-p-*-iso10646-1" "-adobe-helvetica-bold-r-normal-*-10-*-*-*-p-*-iso10646-1" "-adobe-helvetica-bold-r-normal-*-10-*-*-*-p-*-iso10646-1"))
     (foreground . ("grey10" "black"))
     (y-justify . ,vertical-justification)
     (x-justify . ,text-justifier)
     (right-edge . 114)
     (top-edge . -22)
     (class . title))

    ((background . ,top-title-left)
     (left-edge . 9)
     (top-edge . -22)
     (class . title))

    ((background . ,top-title-right)
     (right-edge . 113)
     (top-edge . -22)
     (class . title))

    ((background . ,top-right)
     (right-edge . -8)
     (top-edge . -22)
     (class . title))

    ((background . ,top-over)
     (right-edge . 50)
     (top-edge . 0)
     (class . top-border))

    ((background . ,right-tail)
     (right-edge . -20)
     (top-edge . -21)
     (class . top-right-corner))

    ((background . ,button-exit-images)
     (right-edge . -6)
     (top-edge . -26)
     (class . close-button))

    ((background . ,button-max-images)
     (right-edge . 18)
     (top-edge . -26)
     (class . maximize-button))

    ((background . ,button-icon-images)
     (right-edge . 42)
     (top-edge . -26)
     (class . iconify-button))

    ((background . ,button-menu-images)
     (right-edge . 69)
     (top-edge . -26)
     (class . menu-button))

    ((background . ,button-shade-images)
     (right-edge . 93)
     (top-edge . -26)
     (class . shade-button))

))

(define normal-frame
  `(,@common-frame-parts
    ((background . ,right)
     (right-edge . -8)
     (top-edge . 184)
     (bottom-edge . 46)
     (class . right-border))

    ((background . ,right-top)
     (right-edge . -8)
     (top-edge . 0)
     (class . right-border))

    ((background . ,right-middle)
     (right-edge . -8)
     (top-edge . 81)
     (class . right-border))

    ((background . ,right-grow-top)
     (right-edge . -8)
     (bottom-edge . 9)
     (class . right-border))

    ((background . ,right-grow)
     (right-edge . -12)
     (bottom-edge . -1)
     (class . bottom-right-corner))

    ((background . ,left-middle)
     (left-edge . -8)
     (top-edge . 75)
     (bottom-edge . 44)
     (class . left-border))

    ((background . ,left-top)
     (left-edge . -8)
     (top-edge . 1)
     (class . left-border))

    ((background . ,left-grow-top)
     (left-edge . -8)
     (bottom-edge . 8)
     (class . left-border))

    ((background . ,left-grow)
     (left-edge . -11)
     (bottom-edge . -1)
     (class . bottom-left-corner))

    ((background . ,bottom-line)
     (left-edge . -11)
     (right-edge . -12)
     (bottom-edge . -1)
     (class . bottom-border))
))


(define shaped-frame
  `(,@common-frame-parts

    ((background . ,right-grow)
     (right-edge . -11)
     (top-edge . -1)
     (class . bottom-right-corner))

    ((background . ,left-grow)
     (left-edge . -11)
     (top-edge . 0)
     (class . bottom-left-border))
))
     
;; misc stuff

(define (rebuild-all)
  (rebuild-frames-with-style 'aCOW))

(define (reframe-all)
  (reframe-windows-with-style 'aCOW))

(define (color-changed)
  (recolor-all)
  (reframe-all))

;; initialization

(define initialize-gtkrc
  (let ((done nil))
    (lambda ()
      (unless done
	(require 'gtkrc)
	;; recolour everything when the GTK theme changes
	(gtkrc-call-after-changed color-changed)
	(setq done t)))))

;; setup the initial colours

(add-frame-style 'aCOW
    (lambda (w type)
	(case type
	    ((default) normal-frame)
	    ((transient) normal-frame)
	    ((shaped) shaped-frame)
	    ((shaped-transient) shaped-frame)
	)))
	
(call-after-property-changed
     'WM_NAME (lambda ()
           (rebuild-frames-with-style 'aCOW))))